@@ -115,6 +115,10 @@ public:
     return KernelMapping(start(), end(), fsname_, device_, inode_, prot, flags_,
                          offset);
   }
+  KernelMapping set_fsname(const std::string& name) const {
+    return KernelMapping(start(), end(), name, device_, inode_, prot_, flags_,
+                         offset);
+  }
 
   /**
    * Dump a representation of |this| to a string in a format
@@ -273,6 +277,8 @@ public:
       new (this) Mapping(other);
       return *this;
     }
+    Mapping subrange(MemoryRange range,
+                     std::function<KernelMapping(const KernelMapping&)> f);
 
     const KernelMapping map;
     // The corresponding KernelMapping in the recording. During recording,
@@ -302,7 +308,9 @@ public:
       // This mapping is used for syscallbuf patch stubs
       IS_PATCH_STUBS = 0x4,
       // This mapping is the rr page
-      IS_RR_PAGE = 0x8
+      IS_RR_PAGE = 0x8,
+      // This mapping is the rr vdso page
+      IS_RR_VDSO_PAGE = 0x10,
     };
     uint32_t flags;
   };
@@ -367,6 +375,10 @@ public:
   remote_ptr<void> interp_base() const { return interp_base_; }
   void set_interp_base(remote_ptr<void> base) { interp_base_ = base; }
 
+  // Set anonymous region name as per PR_SET_VMA_ANON_NAME.
+  // Stops at the first unmapped memory page.
+  void set_anon_name(Task* t, MemoryRange range, const std::string* name);
+
   /**
    * Assuming the last retired instruction has raised a SIGTRAP
    * and might be a breakpoint trap instruction, return the type
@@ -759,12 +771,12 @@ public:
   static remote_ptr<uint8_t> rr_page_record_ff_bytes() { return RR_PAGE_FF_BYTES; }
 
   /**
-   * Locate a syscall instruction in t's VDSO.
+   * Locate a syscall instruction in t's VDSO (the real one, not our fake one).
    * This gives us a way to execute remote syscalls without having to write
    * a syscall instruction into executable tracee memory (which might not be
    * possible with some kernels, e.g. PaX).
    */
-  remote_code_ptr find_syscall_instruction(Task* t);
+  remote_code_ptr find_syscall_instruction_in_vdso(Task* t);
 
   /**
    * Task |t| just forked from this address space. Apply dont_fork and
@@ -867,6 +879,7 @@ public:
   };
 
   void map_rr_page(AutoRemoteSyscalls& remote);
+  static std::vector<uint8_t> read_rr_page_for_recording(SupportedArch arch);
   struct UnmapOptions {
     bool exclude_vdso_vvar;
     UnmapOptions() : exclude_vdso_vvar(false) {}
@@ -922,6 +935,7 @@ private:
   void populate_address_space(Task* t);
 
   void unmap_internal(Task* t, remote_ptr<void> addr, ssize_t num_bytes);
+  void update_syscall_ips(Task* t);
 
   bool update_watchpoint_value(const MemoryRange& range,
                                Watchpoint& watchpoint);
