@@ -531,7 +531,7 @@ struct fd_message {
   }
   remote_ptr<int> remote_cmsgdata() {
     return REMOTE_PTR_FIELD(remote_this(), cmsgbuf).as_int() +
-      (uintptr_t)Arch::cmsg_data(NULL);
+      sizeof(typename Arch::cmsghdr);
   }
 };
 
@@ -586,8 +586,8 @@ static long child_recvmsg(AutoRemoteSyscalls& remote, int child_sock) {
     sizeof(msg), &msg, &ok);
 
   if (!ok) {
-    ASSERT(remote.task(), errno == ESRCH) << "Error writing " << remote_buf.get()
-        << " in " << remote.task()->tid;
+    ASSERT(remote.task(), errno == ESRCH || errno == EIO)
+        << "Error writing " << remote_buf.get() << " in " << remote.task()->tid;
     LOG(debug) << "Failed to write memory";
     return -ESRCH;
   }
@@ -603,7 +603,7 @@ static long child_recvmsg(AutoRemoteSyscalls& remote, int child_sock) {
   }
   int their_fd = remote.task()->read_mem(msg.remote_cmsgdata(), &ok);
   if (!ok) {
-    ASSERT(remote.task(), errno == ESRCH);
+    ASSERT(remote.task(), errno == ESRCH || errno == EIO);
     LOG(debug) << "Failed to read msg";
     return -ESRCH;
   }
@@ -691,6 +691,10 @@ template <typename Arch> ScopedFd AutoRemoteSyscalls::retrieve_fd_arch(int fd) {
     if (ret.is_open()) {
       return ret;
     }
+    if (errno == EBADF) {
+      // This can happen when the child was unexpectedly killed.
+      return ret;
+    }
     ASSERT(t, errno == ENOSYS) << "Failed in pidfd_getfd errno=" << errno_name(errno);
   }
 
@@ -845,7 +849,7 @@ void AutoRemoteSyscalls::check_syscall_result(long ret, int syscallno, bool allo
     string extra_msg;
     if (is_open_syscall(syscallno, arch())) {
       extra_msg = " opening " + t->read_c_str(t->regs().arg1());
-    } else if (is_openat_syscall(syscallno, arch())) {
+    } else if (is_openat_syscall(syscallno, arch()) || is_openat2_syscall(syscallno, arch())) {
       extra_msg = " opening " + t->read_c_str(t->regs().arg2());
     } else if (is_mremap_syscall(syscallno, arch()) ||
                is_mmap_syscall(syscallno, arch())) {
