@@ -5,6 +5,8 @@
 
 #include <string>
 
+#include "GdbServerConnection.h"
+
 namespace rr {
 
 class GdbCommand;
@@ -27,12 +29,12 @@ public:
    * should of been striped already.
    */
   static std::string process_command(GdbServer& gdb_server, Task* t,
-                                     std::string payload);
+                                     const GdbRequest::RRCmd& rr_cmd);
 
   static GdbCommand* command_for_name(const std::string& name);
 
   /**
-   * Special return value for commands that immediatly end a diversion session
+   * Special return value for commands that immediately end a diversion session
    */
   static std::string cmd_end_diversion() {
     return std::string("RRCmd_EndDiversion");
