@@ -1056,9 +1056,19 @@ const syscall_patch_hook* Monkeypatcher::find_syscall_hook(RecordTask* t,
       for (size_t i = buf_valid_start_offset; i + 2 <= buf_valid_end_offset; ++i) {
         uint8_t b = bytes[i];
         // Check for short conditional or unconditional jump
-        if (b == 0xeb || (b >= 0x70 && b < 0x80)) {
-          int offset_from_instruction_end = (int)i + 2 + (int8_t)bytes[i + 1] -
-              (LOOK_BACK + instruction_length);
+        int branch_instruction_len = 0;
+        int32_t branch_offset = 0;
+        if (b == 0xeb || b == 0xe3 || (b >= 0x70 && b < 0x80)) {
+          branch_instruction_len = 2;
+          branch_offset = (int8_t)bytes[i + 1];
+        } else if (b == 0x0f && i + 6 <= buf_valid_end_offset &&
+                   (bytes[i + 1] >= 0x80 && bytes[i + 1] < 0x90)) {
+          branch_instruction_len = 6;
+          memcpy(&branch_offset, bytes + i + 2, 4);
+        }
+        if (branch_instruction_len) {
+          int offset_from_instruction_end = (int)i + branch_instruction_len +
+              branch_offset - (LOOK_BACK + instruction_length);
           if (hook.flags & PATCH_SYSCALL_INSTRUCTION_IS_LAST) {
             if (hook.flags & PATCH_IS_MULTIPLE_INSTRUCTIONS) {
               found_potential_interfering_branch =
@@ -1506,12 +1516,11 @@ static void set_and_record_bytes(RecordTask* t, ElfReader& reader,
  * register so that CPU-specific behaviors involving that register don't leak
  * into stack memory.
  */
-static void patch_dl_runtime_resolve(Monkeypatcher& patcher,
-                                     RecordTask* t, ElfReader& reader,
-                                     uintptr_t elf_addr,
-                                     remote_ptr<void> map_start,
-                                     size_t map_size,
-                                     size_t map_offset) {
+void Monkeypatcher::patch_dl_runtime_resolve(RecordTask* t, ElfReader& reader,
+                                             uintptr_t elf_addr,
+                                             remote_ptr<void> map_start,
+                                             size_t map_size,
+                                             size_t map_offset) {
   if (t->arch() != x86_64) {
     return;
   }
@@ -1525,39 +1534,27 @@ static void patch_dl_runtime_resolve(Monkeypatcher& patcher,
   uint8_t *impl_start = impl;
   t->read_bytes(addr, impl);
   if (X64EndBr::match(impl) || X86EndBr::match(impl)) {
-    assert(X64EndBr::size == X86EndBr::size);
+    static_assert(X64EndBr::size == X86EndBr::size, "EndBr patch size mismatch");
     LOG(debug) << "Starts with endbr, skipping";
     addr += X64EndBr::size;
     impl_start += X64EndBr::size;
   }
 
+  static_assert(X64DLRuntimeResolve::size == X64DLRuntimeResolve2::size,
+                "DLRuntimeResolve patch size mismatch");
   if (!X64DLRuntimeResolve::match(impl_start) &&
       !X64DLRuntimeResolve2::match(impl_start)) {
     LOG(warn) << "_dl_runtime_resolve implementation doesn't look right";
     return;
   }
 
-  uint8_t call_patch[X64CallMonkeypatch::size];
-  // We're patching in a relative call, so we need to compute the offset from
-  // the end of the call to our actual destination.
+  vector<uint8_t> bytes(impl_start, impl_start + X64DLRuntimeResolve::size);
   auto call_patch_start = addr.cast<uint8_t>();
-  auto call_patch_end = call_patch_start + sizeof(call_patch);
+  saved_dl_runtime_resolve_code[call_patch_start] = std::move(bytes);
 
-  remote_ptr<uint8_t> extended_call_start =
-      allocate_extended_jump_x86ish<X64DLRuntimeResolvePrelude>(
-          t, patcher.extended_jump_pages, call_patch_end);
-  if (extended_call_start.is_null()) {
-    return;
-  }
-  uint8_t stub_patch[X64DLRuntimeResolvePrelude::size];
-  X64DLRuntimeResolvePrelude::substitute(stub_patch);
-  write_and_record_bytes(t, extended_call_start, stub_patch);
-
-  intptr_t call_offset = extended_call_start - call_patch_end;
-  int32_t call_offset32 = (int32_t)call_offset;
-  ASSERT(t, call_offset32 == call_offset)
-      << "allocate_extended_jump_x86ish didn't work";
-  X64CallMonkeypatch::substitute(call_patch, call_offset32);
+  uint8_t call_patch[X64AbsoluteIndirectCallMonkeypatch::size];
+  X64AbsoluteIndirectCallMonkeypatch::substitute(call_patch,
+      RR_DL_RUNTIME_RESOLVE_CLEAR_FIP);
   write_and_record_bytes(t, call_patch_start, call_patch);
 
   // pad with NOPs to the next instruction
@@ -1568,6 +1565,28 @@ static void patch_dl_runtime_resolve(Monkeypatcher& patcher,
                        sizeof(nops));
 }
 
+void Monkeypatcher::unpatch_dl_runtime_resolves(RecordTask* t) {
+  for (auto entry : saved_dl_runtime_resolve_code) {
+    remote_ptr<uint8_t> addr = entry.first;
+    uint8_t impl[X64DLRuntimeResolve::size];
+    bool ok = true;
+    t->read_bytes_helper(addr, sizeof(impl), impl, &ok);
+    if (!ok) {
+      LOG(warn) << "dl_runtime_resolve code has gone!";
+      continue;
+    }
+    uint8_t call_patch[X64AbsoluteIndirectCallMonkeypatch::size];
+    X64AbsoluteIndirectCallMonkeypatch::substitute(call_patch,
+      RR_PAGE_ADDR - PRELOAD_LIBRARY_PAGE_SIZE);
+    if (memcmp(impl, call_patch, sizeof(call_patch))) {
+      LOG(warn) << "dl_runtime_resolve code has changed!";
+      continue;
+    }
+    write_and_record_mem(t, addr, entry.second.data(), entry.second.size());
+  }
+  saved_dl_runtime_resolve_code.clear();
+}
+
 static bool file_may_need_instrumentation(const AddressSpace::Mapping& map) {
   size_t file_part = map.map.fsname().rfind('/');
   if (file_part == string::npos) {
@@ -1647,7 +1666,7 @@ void Monkeypatcher::patch_after_mmap(RecordTask* t, remote_ptr<void> start,
           (syms.is_name(i, "_dl_runtime_resolve_fxsave") ||
            syms.is_name(i, "_dl_runtime_resolve_xsave") ||
            syms.is_name(i, "_dl_runtime_resolve_xsavec"))) {
-        patch_dl_runtime_resolve(*this, t, reader, syms.addr(i), start, size,
+        patch_dl_runtime_resolve(t, reader, syms.addr(i), start, size,
                                  offset_bytes);
       }
     }
