@@ -65,10 +65,12 @@ static CpuMicroarch compute_cpu_microarch() {
     case 0x506f0:
     case 0x706a0:
     case 0x506c0:
-    case 0x906c0:
       return IntelGoldmont;
+    case 0x906c0:
+      return IntelTremont;
     case 0x706e0:
     case 0x606a0:
+    case 0x80660:
       return IntelIcelake;
     case 0x806c0:
     case 0x806d0:
@@ -85,9 +87,13 @@ static CpuMicroarch compute_cpu_microarch() {
     case 0x906a0:
       return IntelAlderlake;
     case 0xb0670:
+    case 0xb06a0:
+    case 0xb06f0:
       return IntelRaptorlake;
     case 0x806f0:
       return IntelSapphireRapid;
+    case 0xc06f0:
+      return IntelEmeraldRapid;
     case 0x30f00:
       return AMDF15R30;
     case 0x00f10: // Naples, Whitehaven, Summit Ridge, Snowy Owl (Zen), Milan (Zen 3) (UNTESTED)
@@ -110,6 +116,8 @@ static CpuMicroarch compute_cpu_microarch() {
     case 0x50f00: // Cezanne (Zen 3)
     case 0x40f40: // Rembrandt (Zen 3+)
     case 0x60f10: // Raphael (Zen 4)
+    case 0x70f40: // Phoenix (Zen 4)
+    case 0x70f50: // Hawk Point (Zen 4)
       if (ext_family == 0xa) {
         return AMDZen;
       }
