@@ -71,7 +71,8 @@ public:
       bool use_audit = false,
       bool unmap_vdso = false,
       bool force_asan_active = false,
-      bool force_tsan_active = false);
+      bool force_tsan_active = false,
+      bool intel_pt = false);
 
   ~RecordSession() override;
 
@@ -142,8 +143,6 @@ public:
 
   TraceWriter& trace_writer() { return trace_out; }
 
-  virtual void on_destroy(Task* t) override;
-
   Scheduler& scheduler() { return scheduler_; }
 
   SeccompFilterRewriter& seccomp_filter_rewriter() {
@@ -208,6 +207,8 @@ public:
    */
   void forward_SIGTERM();
 
+  void on_destroy_record_task(RecordTask* t);
+
 private:
   RecordSession(const std::string& exe_path,
                 const std::vector<std::string>& argv,
@@ -219,7 +220,8 @@ private:
                 const std::string& output_trace_dir,
                 const TraceUuid* trace_id,
                 bool use_audit,
-                bool unmap_vdso);
+                bool unmap_vdso,
+                bool intel_pt);
 
   virtual void on_create(Task* t) override;
 
@@ -232,6 +234,10 @@ private:
                              RecordResult* step_result,
                              SupportedArch syscall_arch);
   void check_initial_task_syscalls(RecordTask* t, RecordResult* step_result);
+  void handle_seccomp_trap(RecordTask* t, StepState* step_state,
+                           uint16_t seccomp_data);
+  void handle_seccomp_errno(RecordTask* t, StepState* step_state,
+                            uint16_t seccomp_data);
   bool handle_ptrace_event(RecordTask** t_ptr, StepState* step_state,
                            RecordResult* result, bool* did_enter_syscall);
   bool handle_signal_event(RecordTask* t, StepState* step_state);
