@@ -261,6 +261,8 @@ RecordTask::~RecordTask() {
 
   // If this was stopped, notify the scheduler.
   set_stopped(false);
+
+  session().on_destroy_record_task(this);
 }
 
 void RecordTask::record_exit_event(WriteChildTid write_child_tid) {
@@ -1736,7 +1738,7 @@ void RecordTask::record_remote(remote_ptr<void> addr, ssize_t num_bytes) {
   bool ok = true;
   auto buf = read_mem(addr.cast<uint8_t>(), num_bytes, &ok);
   if (!ok) {
-    // Tracee probably died unexpectely. This should only happen
+    // Tracee probably died unexpectedly. This should only happen
     // due to SIGKILL racing with our PTRACE_CONT.
     if (!vm()->find_other_thread_group(this) &&
         vm()->range_is_private_mapping(MemoryRange(addr, num_bytes))) {
@@ -1992,6 +1994,14 @@ void RecordTask::record_event(Event ev, FlushSyscallbuf flush,
   if (should_checksum(ev, current_time)) {
     checksum_process_memory(this, current_time);
   }
+  if (!ev.has_ticks_slop()) {
+    // Only associate PT data with events whose timing is
+    // exactly the same between recording and replay.
+    PTData pt_data = hpc.extract_intel_pt_data();
+    if (!pt_data.data.empty()) {
+      write_pt_data(this, current_time, pt_data.data);
+    }
+  }
 
   if (trace_writer().clear_fip_fdp()) {
     const ExtraRegisters* maybe_extra = extra_regs_fallible();
@@ -2008,7 +2018,7 @@ void RecordTask::record_event(Event ev, FlushSyscallbuf flush,
       registers = &regs();
     }
     if (ev.record_extra_regs()) {
-      extra_registers = &extra_regs();
+      extra_registers = extra_regs_fallible();
     }
   }
 
@@ -2326,4 +2336,79 @@ bool RecordTask::post_vm_clone(CloneReason reason, int flags, Task* origin) {
   return false;
 };
 
+bool RecordTask::try_grow_map(remote_ptr<void> addr) {
+  if (vm()->has_mapping(addr)) {
+    LOG(debug) << "try_grow_map " << addr << ": address already mapped";
+    return false;
+  }
+  auto maps = vm()->maps_starting_at(floor_page_size(addr));
+  auto it = maps.begin();
+  if (it == maps.end()) {
+    LOG(debug) << "try_grow_map " << addr << ": no later map to grow downward";
+    return false;
+  }
+  if (!(it->map.flags() & MAP_GROWSDOWN)) {
+    LOG(debug) << "try_grow_map " << addr << ": map is not MAP_GROWSDOWN ("
+               << it->map << ")";
+    return false;
+  }
+  if (addr >= page_size() && vm()->has_mapping(addr - page_size())) {
+    LOG(debug) << "try_grow_map " << addr << ": address would be in guard page";
+    return false;
+  }
+  remote_ptr<void> limit_bottom;
+#if defined (__i386__)
+  struct rlimit stack_limit;
+  int ret = prlimit(tid, RLIMIT_STACK, NULL, &stack_limit);
+#else
+  struct rlimit64 stack_limit;
+  int ret = syscall(__NR_prlimit64, tid, RLIMIT_STACK, (void*)0, &stack_limit);
+#endif
+  if (ret >= 0 && stack_limit.rlim_cur != RLIM_INFINITY) {
+    limit_bottom = ceil_page_size(it->map.end() - stack_limit.rlim_cur);
+    if (limit_bottom > addr) {
+      LOG(debug) << "try_grow_map " << addr << ": RLIMIT_STACK exceeded";
+      return false;
+    }
+  }
+
+  // Try to grow by 64K at a time to reduce signal frequency.
+  auto new_start = floor_page_size(addr);
+  static const uintptr_t grow_size = 0x10000;
+  if (it->map.start().as_int() >= grow_size) {
+    auto possible_new_start = std::max(
+        limit_bottom, std::min(new_start, it->map.start() - grow_size));
+    // Ensure that no mapping exists between possible_new_start - page_size()
+    // and new_start. If there is, possible_new_start is not valid, in which
+    // case we just abandon the optimization.
+    if (possible_new_start >= page_size() &&
+        !vm()->has_mapping(possible_new_start - page_size()) &&
+        vm()->maps_starting_at(possible_new_start - page_size())
+                .begin()
+                ->map.start() == it->map.start()) {
+      new_start = possible_new_start;
+    }
+  }
+  LOG(debug) << "try_grow_map " << addr << ": trying to grow map " << it->map;
+
+  {
+    AutoRemoteSyscalls remote(this, AutoRemoteSyscalls::DISABLE_MEMORY_PARAMS);
+    remote.infallible_mmap_syscall_if_alive(
+        new_start, it->map.start() - new_start, it->map.prot(),
+        (it->map.flags() & ~MAP_GROWSDOWN) | MAP_ANONYMOUS | MAP_FIXED, -1, 0);
+  }
+
+  KernelMapping km =
+      vm()->map(this, new_start, it->map.start() - new_start, it->map.prot(),
+                it->map.flags() | MAP_ANONYMOUS, 0, string(),
+                KernelMapping::NO_DEVICE, KernelMapping::NO_INODE);
+  trace_writer().write_mapped_region(this, km, km.fake_stat(), km.fsname(), vector<TraceRemoteFd>());
+  // No need to flush syscallbuf here. It's safe to map these pages "early"
+  // before they're really needed.
+  record_event(Event::grow_map(), RecordTask::DONT_FLUSH_SYSCALLBUF);
+  LOG(debug) << "try_grow_map " << addr << ": extended map "
+             << vm()->mapping_of(addr).map;
+  return true;
+}
+
 } // namespace rr
