@@ -48,10 +48,12 @@ Session::Session()
       next_task_serial_(1),
       rrcall_base_(RR_CALL_BASE),
       syscallbuf_fds_disabled_size_(SYSCALLBUF_FDS_DISABLED_SIZE),
+      syscallbuf_hdr_size_(sizeof(syscallbuf_hdr)),
       syscall_seccomp_ordering_(PTRACE_SYSCALL_BEFORE_SECCOMP_UNKNOWN),
       ticks_semantics_(PerfCounters::default_ticks_semantics()),
       done_initial_exec_(false),
-      visible_execution_(true) {
+      visible_execution_(true),
+      intel_pt_(false) {
   LOG(debug) << "Session " << this << " created";
 }
 
@@ -64,19 +66,21 @@ Session::~Session() {
   }
 }
 
-Session::Session(const Session& other) {
-  statistics_ = other.statistics_;
-  next_task_serial_ = other.next_task_serial_;
-  done_initial_exec_ = other.done_initial_exec_;
-  rrcall_base_ = other.rrcall_base_;
-  syscallbuf_fds_disabled_size_ = other.syscallbuf_fds_disabled_size_;
-  visible_execution_ = other.visible_execution_;
-  tracee_socket = other.tracee_socket;
-  tracee_socket_receiver = other.tracee_socket_receiver;
-  tracee_socket_fd_number = other.tracee_socket_fd_number;
-  ticks_semantics_ = other.ticks_semantics_;
-  original_affinity_ = other.original_affinity_;
-}
+Session::Session(const Session& other)
+    : statistics_(other.statistics_),
+      tracee_socket(other.tracee_socket),
+      tracee_socket_receiver(other.tracee_socket_receiver),
+      tracee_socket_fd_number(other.tracee_socket_fd_number),
+      next_task_serial_(other.next_task_serial_),
+      rrcall_base_(other.rrcall_base_),
+      syscallbuf_fds_disabled_size_(other.syscallbuf_fds_disabled_size_),
+      syscallbuf_hdr_size_(other.syscallbuf_hdr_size_),
+      syscall_seccomp_ordering_(other.syscall_seccomp_ordering_),
+      ticks_semantics_(other.ticks_semantics_),
+      original_affinity_(other.original_affinity_),
+      done_initial_exec_(other.done_initial_exec_),
+      visible_execution_(other.visible_execution_),
+      intel_pt_(other.intel_pt_) {}
 
 void Session::on_create(ThreadGroup* tg) { thread_group_map_[tg->tguid()] = tg; }
 void Session::on_destroy(ThreadGroup* tg) {
@@ -217,7 +221,7 @@ void Session::kill_all_tasks() {
     /* We delete tasks in two passes. First, we kill
      * every non-thread-group-leader, then we kill every group leader.
      * Linux expects threads group leaders to survive until the last
-     * member of the thread group has exited, so we accomodate that.
+     * member of the thread group has exited, so we accommodate that.
      */
     for (auto& v : task_map) {
       Task* t = v.second;
@@ -651,9 +655,8 @@ static vector<uint8_t> capture_syscallbuf(const AddressSpace::Mapping& m,
     // so just record the entire buffer. This should not be common.
     data_size = m.map.size();
   } else {
-    data_size = clone_leader->read_mem(
-                    REMOTE_PTR_FIELD(syscallbuf_hdr, num_rec_bytes)) +
-                sizeof(struct syscallbuf_hdr);
+    data_size = clone_leader->read_mem(REMOTE_PTR_FIELD(syscallbuf_hdr, num_rec_bytes)) +
+        clone_leader->session().syscallbuf_hdr_size();
   }
   return clone_leader->read_mem(start, data_size);
 }
@@ -759,6 +762,10 @@ static bool set_cpu_affinity(int cpu) {
 }
 
 void Session::do_bind_cpu() {
+  if (intel_pt_) {
+    PerfCounters::start_pt_copy_thread();
+  }
+
   sched_getaffinity(0, sizeof(original_affinity_), &original_affinity_);
 
   int cpu_index = this->cpu_binding();
@@ -791,4 +798,8 @@ void Session::do_bind_cpu() {
   }
 }
 
+bool Session::mark_stdio() const {
+  return visible_execution_ && Flags::get().mark_stdio;
+}
+
 } // namespace rr
