@@ -11,9 +11,9 @@
 namespace rr {
 
 Switchable StdioMonitor::will_write(Task* t) {
-  if (Flags::get().mark_stdio && t->session().visible_execution()) {
+  if (t->session().mark_stdio()) {
     char buf[256];
-    snprintf(buf, sizeof(buf) - 1, "[rr %d %d]", t->tgid(), t->trace_time());
+    snprintf(buf, sizeof(buf) - 1, "[rr %d %ld]", t->tgid(), t->trace_time());
     ssize_t len = strlen(buf);
     if (write(original_fd, buf, len) != len) {
       ASSERT(t, false) << "Couldn't write to " << original_fd;
@@ -25,17 +25,15 @@ Switchable StdioMonitor::will_write(Task* t) {
 
 void StdioMonitor::did_write(Task* t, const std::vector<Range>& ranges,
                              LazyOffset&) {
-  if (!t->session().is_replaying()) {
+  ReplaySession* replay_session = t->session().as_replay();
+  if (!replay_session || !replay_session->echo_stdio()) {
     return;
   }
-  auto rt = static_cast<ReplayTask*>(t);
-  if (rt->session().flags().redirect_stdio && rt->session().visible_execution()) {
-    for (auto& r : ranges) {
-      auto bytes = t->read_mem(r.data.cast<uint8_t>(), r.length);
-      if (bytes.size() !=
-          (size_t)write(original_fd, bytes.data(), bytes.size())) {
-        ASSERT(t, false) << "Couldn't write to " << original_fd;
-      }
+  for (auto& r : ranges) {
+    auto bytes = t->read_mem(r.data.cast<uint8_t>(), r.length);
+    if (bytes.size() !=
+        (size_t)write(original_fd, bytes.data(), bytes.size())) {
+      ASSERT(t, false) << "Couldn't write to " << original_fd;
     }
   }
 }
