@@ -139,6 +139,8 @@ class Task {
 public:
   typedef std::vector<WatchConfig> HardwareWatchpoints;
 
+  ReplayTask* as_replay();
+
   /**
    * Ptrace-detach the task.
    */
@@ -178,10 +180,7 @@ public:
    */
   void finish_emulated_syscall();
 
-  size_t syscallbuf_data_size() {
-    return read_mem(REMOTE_PTR_FIELD(syscallbuf_child, num_rec_bytes)) +
-           sizeof(struct syscallbuf_hdr);
-  }
+  size_t syscallbuf_data_size();
 
   /**
    * Dump attributes of this process, including pending events,
@@ -401,8 +400,6 @@ public:
 
   /**
    * Hook called by `resume_execution`.
-   * Returns `false` if the task is in the process of dying and setup could not
-   * be completed, `true` otherwise.
    */
   virtual void will_resume_execution(ResumeRequest, WaitRequest, TicksRequest,
                                      int /*sig*/) {}
@@ -509,9 +506,6 @@ public:
   /** Return the current regs of this. */
   const Registers& regs() const;
 
-  /** Return the extra registers of this. Asserts if the task died. */
-  const ExtraRegisters& extra_regs();
-
   /** Return the extra registers of this, or null if the task died. */
   const ExtraRegisters* extra_regs_fallible();
 
@@ -601,8 +595,10 @@ public:
   /** Set the tracee's registers to |regs|. Lazy. */
   void set_regs(const Registers& regs);
 
-  /** Ensure registers are flushed back to the underlying task. */
-  void flush_regs();
+  /** Ensure registers are flushed back to the underlying task.
+   *  Returns false if that failed due to the tracee being in
+   *  an unexpected state. */
+  bool flush_regs();
 
   /** Set the tracee's extra registers to |regs|. */
   void set_extra_regs(const ExtraRegisters& regs);
@@ -669,6 +665,8 @@ public:
    */
   virtual void set_stopped(bool stopped) { is_stopped_ = stopped; }
 
+  bool in_injectable_signal_stop() const { return in_injectable_signal_stop_; }
+
   /**
    * Return the status of this as of the last successful wait()/try_wait() call.
    */
@@ -705,7 +703,7 @@ public:
    * events.  |task_time()| returns that "time" wrt this task
    * only.
    */
-  uint32_t trace_time() const;
+  FrameTime trace_time() const;
 
   /**
    * Call this to reset syscallbuf_hdr->num_rec_bytes and zero out the data
@@ -1311,6 +1309,11 @@ protected:
   // True when we've been kicked out of a ptrace-stop via SIGKILL or
   // equivalent.
   bool in_unexpected_exit;
+  // True when the task is stopped in a signal-stop where we can
+  // inject our own signal. Usually equal to wait_status.stop_sig() > 0,
+  // but can be different if an AutoRemoteSyscalls changed our state and
+  // then restored wait_status.
+  bool in_injectable_signal_stop_;
   /* True when the seccomp filter has been enabled via prctl(). This happens
    * in the first system call issued by the initial tracee (after it returns
    * from kill(SIGSTOP) to synchronize with the tracer). */
