@@ -1,5 +1,10 @@
 /* -*- Mode: C++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
+#ifndef _GNU_SOURCE
+// For utsname::domainname
+#define _GNU_SOURCE 1
+#endif
+
 #include "TraceStream.h"
 
 #include <capnp/message.h>
@@ -8,6 +13,7 @@
 #include <limits.h>
 #include <sched.h>
 #include <sys/file.h>
+#include <sys/utsname.h>
 #include <sys/wait.h>
 #include <sysexits.h>
 #include <dirent.h>
@@ -230,6 +236,9 @@ static kj::ArrayPtr<const capnp::byte> str_to_data(const string& str) {
 }
 
 static string data_to_str(const kj::ArrayPtr<const capnp::byte>& data) {
+  if (!data.begin()) {
+    return string();
+  }
   if (memchr(data.begin(), 0, data.size())) {
     FATAL() << "Invalid string: contains null character";
   }
@@ -491,10 +500,12 @@ void TraceWriter::write_frame(RecordTask* t, const Event& ev,
       auto data = syscall.initExtra();
       if (e.write_offset >= 0) {
         data.setWriteOffset(e.write_offset);
-      } else if (e.exec_fds_to_close.size()) {
+      }
+      if (!e.exec_fds_to_close.empty()) {
         data.setExecFdsToClose(kj::ArrayPtr<const int>(
             e.exec_fds_to_close.data(), e.exec_fds_to_close.size()));
-      } else if (e.opened.size()) {
+      }
+      if (!e.opened.empty()) {
         auto open = data.initOpenedFds(e.opened.size());
         for (size_t i = 0; i < e.opened.size(); ++i) {
           auto o = open[i];
@@ -504,13 +515,23 @@ void TraceWriter::write_frame(RecordTask* t, const Event& ev,
           o.setDevice(opened.device);
           o.setInode(opened.inode);
         }
-      } else if (e.socket_addrs) {
+      }
+      if (e.socket_addrs) {
         auto addrs = data.initSocketAddrs();
         auto localAddr = (*e.socket_addrs.get())[0];
         auto remoteAddr = (*e.socket_addrs.get())[1];
         addrs.setLocalAddr(Data::Reader(reinterpret_cast<uint8_t*>(&localAddr), sizeof(localAddr)));
         addrs.setRemoteAddr(Data::Reader(reinterpret_cast<uint8_t*>(&remoteAddr), sizeof(remoteAddr)));
       }
+      if (!e.madvise_ranges.empty()) {
+        auto ranges = data.initMadviseRanges(e.madvise_ranges.size());
+        for (size_t i = 0; i < e.madvise_ranges.size(); ++i) {
+          auto r = ranges[i];
+          auto mr = e.madvise_ranges[i];
+          r.setStart(mr.start().as_int());
+          r.setEnd(mr.end().as_int());
+        }
+      }
       break;
     }
     default:
@@ -648,8 +669,10 @@ TraceFrame TraceReader::read_frame(FrameTime skip_before) {
       auto mprotect_records = event.getSyscallbufFlush().getMprotectRecords();
       auto& records = ret.ev.SyscallbufFlush().mprotect_records;
       records.resize(mprotect_records.size() / sizeof(mprotect_record));
-      memcpy(records.data(), mprotect_records.begin(),
-             records.size() * sizeof(mprotect_record));
+      if (records.data()) {
+        memcpy(records.data(), mprotect_records.begin(),
+               records.size() * sizeof(mprotect_record));
+      }
       break;
     }
     case trace::Frame::Event::SYSCALL: {
@@ -706,6 +729,16 @@ TraceFrame TraceReader::read_frame(FrameTime skip_before) {
           memcpy(&(*syscall_ev.socket_addrs.get())[1], remote.begin(), sizeof(NativeArch::sockaddr_storage));
           break;
         }
+        case trace::Frame::Event::Syscall::Extra::MADVISE_RANGES: {
+          auto mrs = data.getMadviseRanges();
+          syscall_ev.madvise_ranges.resize(mrs.size());
+          for (size_t i = 0; i < mrs.size(); ++i) {
+            const auto& mr = mrs[i];
+            syscall_ev.madvise_ranges[i] =
+                MemoryRange(remote_ptr<void>(mr.getStart()), remote_ptr<void>(mr.getEnd()));
+          }
+          break;
+        }
         default:
           FATAL() << "Unknown syscall type";
           break;
@@ -1208,14 +1241,6 @@ void TraceWriter::write_raw_data(const void* d, size_t len) {
   data.write(d, len);
 }
 
-TraceReader::RawData TraceReader::read_raw_data() {
-  RawData d;
-  if (!read_raw_data_for_frame(d)) {
-    FATAL() << "Expected raw data, found none";
-  }
-  return d;
-}
-
 bool TraceReader::read_raw_data_for_frame(RawData& d) {
   if (raw_recs.empty()) {
     return false;
@@ -1334,7 +1359,10 @@ TraceWriter::TraceWriter(const std::string& file_name,
       has_cpuid_faulting_(false),
       xsave_fip_fdp_quirk_(false),
       fdp_exception_only_quirk_(false),
-      clear_fip_fdp_(false) {
+      clear_fip_fdp_(false),
+      supports_file_data_cloning_(false),
+      chaos_mode(false)
+       {
   this->ticks_semantics_ = ticks_semantics_;
 
   for (Substream s = SUBSTREAM_FIRST; s < SUBSTREAM_COUNT; ++s) {
@@ -1412,6 +1440,7 @@ void TraceWriter::close(CloseStatus status, const TraceUuid* uuid) {
   header.setPreloadThreadLocalsRecorded(true);
   header.setRrcallBase(syscall_number_for_rrcall_init_preload(x86_64));
   header.setSyscallbufFdsDisabledSize(SYSCALLBUF_FDS_DISABLED_SIZE);
+  header.setSyscallbufHdrSize(sizeof(syscallbuf_hdr));
 
   header.setNativeArch(to_trace_arch(NativeArch::arch()));
   if (NativeArch::is_x86ish())
@@ -1452,6 +1481,21 @@ void TraceWriter::close(CloseStatus status, const TraceUuid* uuid) {
   header.setRuntimePageSize(page_size());
   header.setPreloadLibraryPageSize(PRELOAD_LIBRARY_PAGE_SIZE);
 
+  {
+    struct utsname uname_buf;
+    int ret = uname(&uname_buf);
+    if (ret) {
+      FATAL() << "uname failed";
+    }
+    auto uname_msg = header.initUname();
+    uname_msg.setSysname(str_to_data(uname_buf.sysname));
+    uname_msg.setNodename(str_to_data(uname_buf.nodename));
+    uname_msg.setRelease(str_to_data(uname_buf.release));
+    uname_msg.setVersion(str_to_data(uname_buf.version));
+    uname_msg.setMachine(str_to_data(uname_buf.machine));
+    uname_msg.setDomainname(str_to_data(uname_buf.domainname));
+  }
+
   try {
     writePackedMessageToFd(version_fd, header_msg);
   } catch (...) {
@@ -1598,6 +1642,7 @@ TraceReader::TraceReader(const string& dir)
   ticks_semantics_ = from_trace_ticks_semantics(header.getTicksSemantics());
   rrcall_base_ = header.getRrcallBase();
   syscallbuf_fds_disabled_size_ = header.getSyscallbufFdsDisabledSize();
+  syscallbuf_hdr_size_ = header.getSyscallbufHdrSize();
   required_forward_compatibility_version_ = header.getRequiredForwardCompatibilityVersion();
   quirks_ = 0;
   {
@@ -1658,6 +1703,14 @@ TraceReader::TraceReader(const string& dir)
   exclusion_range_ = MemoryRange(remote_ptr<void>(header.getExclusionRangeStart()),
                                  remote_ptr<void>(header.getExclusionRangeEnd()));
 
+  const auto& uname = header.getUname();
+  uname_.sysname = data_to_str(uname.getSysname());
+  uname_.nodename = data_to_str(uname.getNodename());
+  uname_.release = data_to_str(uname.getRelease());
+  uname_.version = data_to_str(uname.getVersion());
+  uname_.machine = data_to_str(uname.getMachine());
+  uname_.domainname = data_to_str(uname.getDomainname());
+
   // Set the global time at 0, so that when we tick it for the first
   // event, it matches the initial global time at recording, 1.
   global_time = 0;
@@ -1686,6 +1739,7 @@ TraceReader::TraceReader(const TraceReader& other)
   chaos_mode_ = other.chaos_mode_;
   chaos_mode_known_ = other.chaos_mode_known_;
   exclusion_range_ = other.exclusion_range_;
+  uname_ = other.uname_;
   quirks_ = other.quirks_;
   clear_fip_fdp_ = other.clear_fip_fdp_;
   required_forward_compatibility_version_ = other.required_forward_compatibility_version_;
