@@ -3,7 +3,9 @@
 #ifndef RR_KERNEL_SUPPLEMENT_H_
 #define RR_KERNEL_SUPPLEMENT_H_
 
+#ifndef _GNU_SOURCE
 #define _GNU_SOURCE 1
+#endif
 
 #include <linux/capability.h>
 #include <linux/if_tun.h>
@@ -114,6 +116,10 @@ enum _ptrace_get_syscall_info_op {
 #define SYS_SECCOMP 1
 #endif
 
+#ifndef SOL_NETLINK
+#define SOL_NETLINK 270
+#endif
+
 #ifndef PR_GET_SPECULATION_CTRL
 #define PR_GET_SPECULATION_CTRL 52
 #endif
@@ -277,8 +283,11 @@ struct rr_input_mask {
 
 // Defined in the ip_tables header for each protocol, but always to the same,
 // value, so it should be fine to set this here
-#ifndef SO_SET_REPLACE
-#define SO_SET_REPLACE 64
+#ifndef IPT_SO_SET_REPLACE
+#define IPT_SO_SET_REPLACE 64
+#endif
+#ifndef IPV6T_SO_SET_REPLACE
+#define IPV6T_SO_SET_REPLACE 64
 #endif
 
 #ifndef HCIGETDEVLIST
@@ -410,6 +419,9 @@ struct rr_input_mask {
 #ifndef MAP_SYNC
 #define MAP_SYNC  0x80000
 #endif
+#ifndef MAP_FIXED_NOREPLACE
+#define MAP_FIXED_NOREPLACE 0x100000
+#endif
 
 enum {
   BPF_MAP_CREATE,
