@@ -1,11 +1,7 @@
 from util import *
 
 def test_cond(c):
-    send_gdb('cond 1 %s'%c)
-    # check that the condition is evaluated correctly by checking that
-    # we don't break on the negation of the condition
-    send_gdb('cond 2 !(%s)'%c)
-    send_gdb('c')
+    send_gdb('python gdb.execute("cond 1 %s"); gdb.execute("cond 2 !(%s)"); gdb.execute("c")' % (c, c))
     expect_gdb('Breakpoint 1')
 
 send_gdb('b breakpointA')
